/*
 * Copyright 2011 DeepDiff Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package deepdiff.core;

/**
 * Interface to support classes that wish to have their behavior be configurable
 * in a standard way. When using classes that implement this interface, call
 * {@link #addProperty(ConfigProperty)} once for each property defined for the
 * object, and then call {@link #validateProperties()} once after all properties
 * have been added.
 */
public interface Configurable {
    /**
     * Adds a property to the configuration for the object
     *
     * @param property the property to add
     *
     * @throws IllegalConfigException if the specified property is illegal for
     *                                some reason
     */
    void addProperty(ConfigProperty property) throws IllegalConfigException;

    /**
     * Verifies that the configured properties (specified via {@link
     * #addProperty(ConfigProperty)}) are valid together as a complete
     * configuration for the object. This is most commonly used to check for any
     * missing properties, but can also be used to check for duplicate
     * properties, properties that depend on other properties, etc.
     *
     * @throws IllegalConfigException if the properties are illegal for some
     *                                reason
     */
    void validateProperties() throws IllegalConfigException;
}
