/*
 * Copyright 2011 DeepDiff Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package deepdiff.scope;

import java.io.File;
import java.util.Arrays;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import deepdiff.core.DiffPointProcessor;
import deepdiff.core.DiffScope;
import deepdiff.core.DiffUnit;
import deepdiff.core.DiffUnitProcessor;

/**
 * An implementation of {@link DiffScope} that uses two directories (represented as Files) as the
 * root for a recursive directory scan. UNC is supported.
 */
public class FileDiffScope implements DiffScope {
    private static final Logger log = Logger.getLogger(FileDiffScope.class);
    private File root1;
    private File root2;

    /**
     * Initializes a new FileDiffScope object.
     * 
     * @param root1 the root directory on the left
     * @param root2 the root directory on the right
     */
    public FileDiffScope(File root1, File root2) {
        this.root1 = root1;
        this.root2 = root2;
    }

    /**
     * Starts the recursive directory scan. Files are considered {@link DiffUnit}s, as are missing
     * directories. Matching directories are not. All DiffUnits are passed to the specified unit
     * processor using the specified point processor.
     * 
     * @param unitProcessor the unit processor to pass {@link DiffUnit}s to
     * @param pointProcessor the point process to pass to the unit processor
     */
    public void scan(DiffUnitProcessor unitProcessor, DiffPointProcessor pointProcessor) {
        String path = getPath();
        log.debug("Starting to scan scope " + path);
        scan(root1, root2, unitProcessor, pointProcessor);
        log.debug("Completed scan of scope " + path);
    }

    /**
     * Recursively scans a pair of {@link File}s. Files are considered {@link DiffUnit}s, as are
     * missing directories. Matching directories are not. All DiffUnits are passed to the specified
     * unit processor using the specified point processor.
     * 
     * @param f1 the File on the left to scan
     * @param f2 the File on the right to scan
     * @param unitProcessor the unit processor to pass {@link DiffUnit}s to
     * @param pointProcessor the point process to pass to the unit processor
     */
    private void scan(File f1, File f2, DiffUnitProcessor unitProcessor,
            DiffPointProcessor pointProcessor) {
        if (f1.isDirectory() && f2.isDirectory()) {
            // Compare two directories recursively
            SortedSet<String> names = new TreeSet<String>();
            names.addAll(Arrays.asList(f1.list()));
            names.addAll(Arrays.asList(f2.list()));
            for (Iterator<String> it = names.iterator(); it.hasNext();) {
                String name = (String) it.next();
                File c1 = new File(f1, name);
                File c2 = new File(f2, name);
                scan(c1, c2, unitProcessor, pointProcessor);
            }
        } else {
            DiffUnit diffUnit = new FileDiffUnit(this, f1, f2);
            unitProcessor.processDiffUnit(diffUnit, pointProcessor);
        }
    }

    /**
     * Returns the root directory on the left
     * 
     * @return the root directory on the left
     */
    File getLeftRoot() {
        return root1;
    }

    /**
     * Returns the root directory on the right
     * 
     * @return the root directory on the right
     */
    File getRightRoot() {
        return root2;
    }

    /**
     * Returns the path of the scope
     * 
     * @return the path of the scope
     */
    public String getPath() {
        return "[" + root1.getPath() + "," + root2.getPath() + "]";
    }
}
