/*
 * Copyright 2011 DeepDiff Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package deepdiff.scope;

import java.io.IOException;
import java.io.InputStream;

import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.apache.log4j.Logger;

import deepdiff.core.DiffPointProcessor;
import deepdiff.core.DiffScope;
import deepdiff.core.DiffUnitProcessor;

/**
 * An implementation of {@link DiffScope} that compares two Zip files. These Zip
 * files do not need to be physical files; any {@link InputStream} can be used.
 * Thus, it is possible to process Zip files within Zip files.
 */
public class ZipDiffScope implements DiffScope {
    private static final Logger log = Logger.getLogger(ZipDiffScope.class);

    private InputStream is1;
    private InputStream is2;

    private String path;

    /**
     * Initializes a new ZipDiffScope object.
     *
     * @param path the scoped path to the Zip files
     * @param is1 the {@link InputStream} for the Zip file on the left
     * @param is2 the {@link InputStream} for the Zip file on the right
     */
    public ZipDiffScope(String path, InputStream is1, InputStream is2) {
        this.path = path;
        this.is1 = is1;
        this.is2 = is2;
    }

    /**
     * Scans the Zip archive for DiffUnits and processes them
     *
     * @param unitProcessor the unit processor to process the DiffUnits found in
     *                      the scan
     * @param pointProcessor the point processor to pass to the unit processor
     *                       when processing units
     */
    public void scan(DiffUnitProcessor unitProcessor,
                     DiffPointProcessor pointProcessor) {
        log.debug("Starting to scan scope " + path);
        ZipInputStream zis1 = new ZipInputStream(is1);
        ZipInputStream zis2 = new ZipInputStream(is2);
        try {
            ZipEntry e1 = zis1.getNextEntry();
            ZipEntry e2 = zis2.getNextEntry();
            while (e1 != null || e2 != null) {
                if (e1 == null) {
                    ZipDiffUnit diffUnit =
                        new ZipDiffUnit(this, null, zis2, null, e2);
                    unitProcessor.processDiffUnit(diffUnit, pointProcessor);
                    e2 = zis2.getNextEntry();
                } else if (e2 == null) {
                    ZipDiffUnit diffUnit =
                        new ZipDiffUnit(this, zis1, null, e1, null);
                    unitProcessor.processDiffUnit(diffUnit, pointProcessor);
                    e1 = zis1.getNextEntry();
                } else {
                    String entryName1 = e1.getName();
                    String entryName2 = e2.getName();
                    if (entryName1.equals(entryName2)) {
                        ZipDiffUnit diffUnit =
                            new ZipDiffUnit(this, zis1, zis2, e1, e2);
                        unitProcessor.processDiffUnit(diffUnit, pointProcessor);
                        e1 = zis1.getNextEntry();
                        e2 = zis2.getNextEntry();
                    } else {
                        if (entryName1.compareTo(entryName2) < 0) {
                            ZipDiffUnit diffUnit =
                                new ZipDiffUnit(this, zis1, null, e1, null);
                            unitProcessor.processDiffUnit(diffUnit,
                                pointProcessor);
                            e1 = zis1.getNextEntry();
                        } else {
                            ZipDiffUnit diffUnit =
                                new ZipDiffUnit(this, null, zis2, null, e2);
                            unitProcessor.processDiffUnit(diffUnit,
                                pointProcessor);
                            e2 = zis2.getNextEntry();
                        }
                    }
                }
            }
        } catch (IOException ioe) {
            log.error("Failure scanning scope", ioe);
        } finally {
            if (zis1 != null) {
                try {
                    zis1.close();
                } catch (Exception ex) {
                    //Ignore exception on close
                }
            }
            if (zis2 != null) {
                try {
                    zis2.close();
                } catch (Exception ex) {
                    //Ignore exception on close
                }
            }
        }
        log.debug("Completed scan of scope " + path);
    }

    /**
     * Returns the scoped path to the Zip files
     *
     * @return the scoped path to the Zip files
     */
    public String getPath() {
        return path;
    }
}
