/*
 * Copyright 2011 DeepDiff Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package deepdiff.unitprocessor;

import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import deepdiff.core.ConfigProperty;
import deepdiff.core.Configurable;
import deepdiff.core.DiffPoint;
import deepdiff.core.DiffPointProcessor;
import deepdiff.core.DiffScope;
import deepdiff.core.DiffUnit;
import deepdiff.core.DiffUnitProcessor;
import deepdiff.core.DiffUnitProcessorFactory;
import deepdiff.core.IllegalConfigException;
import deepdiff.core.ScopedDiffUnitProcessor;

/**
 * A Configurable DiffUnitProcessor that scans for files, and delegates file comparisons to other
 * {@link DelegatingDiffUnitProcessor}s based on its configuration. It also supports excluding
 * certain paths from being processed.
 */
public class DelegatingDiffUnitProcessor implements ScopedDiffUnitProcessor, Configurable {
    public static final String TYPE_EXCLUDE = "exclude";
    public static final String TYPE_HANDLER = "handler";

    private static final Logger log = Logger.getLogger(DelegatingDiffUnitProcessor.class);
    private Map<String, DiffUnitProcessor> handlers = new LinkedHashMap<String, DiffUnitProcessor>();

    private Collection<Pattern> excludes = new LinkedList<Pattern>();

    public Map<String, DiffUnitProcessor> getHandlers() {
        return Collections.unmodifiableMap(handlers);
    }

    /**
     * Adds a property to the processor configuration
     * 
     * @param property the property to add
     * 
     * @throws IllegalConfigException if the property isn't valid
     */
    public void addProperty(ConfigProperty property) throws IllegalConfigException {
        String type = property.getType();
        if (TYPE_HANDLER.equals(type)) {
            String regex = property.getName();
            String handlerId = property.getValue();
            if (handlers.containsKey(regex)) {
                throw new IllegalConfigException("Multiple handlers are configured for pattern: "
                        + regex);
            }
            DiffUnitProcessor fileHandler = DiffUnitProcessorFactory.get(handlerId);
            handlers.put(regex, fileHandler);
        } else if (TYPE_EXCLUDE.equals(type)) {
            String regex = property.getValue();
            Pattern pattern = Pattern.compile(regex);
            excludes.add(pattern);
        } else {
            throw new IllegalConfigException("Unsupported property type: " + type);
        }
    }

    /**
     * Checks that at least one file handler was configured
     * 
     * @throws IllegalConfigException if no file handlers were configured
     */
    public void validateProperties() throws IllegalConfigException {
        if (handlers.isEmpty()) {
            throw new IllegalConfigException("No properties with type " + TYPE_HANDLER
                    + " specified");
        }
    }

    /**
     * Scans <code>diffUnit</code>. Any missing files/directories are directly turned into
     * {@link DiffPoint}s and passed to <code>processor</code>. File comparisons are delegated to
     * file handlers, other implementations of {@link DiffUnitProcessor}. If the delegate returns a
     * child scope, the child scope will be processed in the same fashion.
     * 
     * @param diffUnit the unit to process
     * @param processor the processor to pass differences to
     * 
     * @return null (no child scopes needed)
     */
    public DiffScope processDiffUnit(DiffUnit diffUnit, DiffPointProcessor processor) {
        String scopedPath = diffUnit.getScopedPath();
        for (Iterator<Pattern> it = excludes.iterator(); it.hasNext();) {
            Pattern pattern = it.next();
            if (pattern.matcher(scopedPath).matches()) {
                log.debug("Skipping unit due to exclude: " + scopedPath);
                return null;
            }
        }
        log.debug("Processing unit " + scopedPath);
        boolean leftExists = diffUnit.leftExists();
        boolean rightExists = diffUnit.rightExists();
        boolean leftIsDir = diffUnit.leftIsDir();
        boolean rightIsDir = diffUnit.rightIsDir();
        if (leftExists && !rightExists) {
            DiffPoint diffPoint = new DiffPoint(diffUnit, "Exists in left only");
            processor.processDiffPoint(diffPoint);
        } else if (rightExists && !leftExists) {
            DiffPoint diffPoint = new DiffPoint(diffUnit, "Exists in right only");
            processor.processDiffPoint(diffPoint);
        } else if (leftIsDir != rightIsDir) {
            DiffPoint diffPoint = new DiffPoint(diffUnit, "Dir/File mismatch");
            processor.processDiffPoint(diffPoint);
        } else if (!leftIsDir && !rightIsDir) {
            boolean foundHandler = false;
            for (Iterator<Map.Entry<String, DiffUnitProcessor>> it = handlers.entrySet().iterator(); it
                    .hasNext();) {
                Map.Entry<String, DiffUnitProcessor> entry = it.next();
                String regex = entry.getKey();
                if (scopedPath.matches(regex)) {
                    DiffUnitProcessor fileHandler = entry.getValue();
                    DiffScope childScope = fileHandler.processDiffUnit(diffUnit, processor);
                    if (childScope != null) {
                        childScope.scan(this, processor);
                    }
                    foundHandler = true;
                    break;
                }
            }
            if (!foundHandler) {
                log.error("No applicable handler found for path: " + scopedPath);
            }
        }
        return null; // No child scopes needed
    }

    @Override
    public DiffScope createScope(String path, InputStream is1, InputStream is2) {
        for (Pattern pattern : excludes) {
            if (pattern.matcher(path).matches()) {
                log.debug("Skipping unit due to exclude: " + path);
                return null;
            }
        }
        for (String regex : handlers.keySet()) {
            if (path.matches(regex)) {
                DiffUnitProcessor unitProcessor = handlers.get(regex);
                if (unitProcessor instanceof ScopedDiffUnitProcessor) {
                    ScopedDiffUnitProcessor sdup = (ScopedDiffUnitProcessor) unitProcessor;
                    return sdup.createScope(path, is1, is2);
                }
            }
        }
        return null;
    }
}
