/*
 * Copyright 2011 DeepDiff Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package deepdiff.unitprocessor;

import java.io.IOException;
import java.io.Serializable;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import org.apache.log4j.Logger;

import deepdiff.core.DiffPoint;
import deepdiff.core.DiffPointProcessor;
import deepdiff.core.DiffScope;
import deepdiff.core.DiffUnit;
import deepdiff.core.DiffUnitProcessor;

/**
 * A DiffUnitProcessor that performs Manifest comparisons on DiffUnits.
 */
public class ManifestCompareDiffUnitProcessor implements DiffUnitProcessor {
    private static final Logger log = Logger.getLogger(ManifestCompareDiffUnitProcessor.class);

    /**
     * Performs a Manifest comparison on <code>diffUnit</code>, and passes any {@link DiffPoint}s
     * found to <code>processor</code>. In each manifest entry (including the main entry), the
     * attributes are iterated through and their values compared.
     * 
     * @param diffUnit the unit to process
     * @param processor the processor to pass differences to
     * 
     * @return null (no child scopes needed)
     */
    public DiffScope processDiffUnit(DiffUnit diffUnit, DiffPointProcessor processor) {
        try {
            Manifest m1 = new Manifest(diffUnit.getLeftInputStream());
            Manifest m2 = new Manifest(diffUnit.getRightInputStream());
            compareAttributes(diffUnit, "[MAIN]", m1.getMainAttributes(), m2.getMainAttributes(),
                    processor);
            Map<String, Attributes> entries1 = m1.getEntries();
            Map<String, Attributes> entries2 = m2.getEntries();
            SortedSet<String> entryKeys = new TreeSet<String>();
            entryKeys.addAll(entries1.keySet());
            entryKeys.addAll(entries2.keySet());
            for (Iterator<String> it = entryKeys.iterator(); it.hasNext();) {
                String entryName = it.next();
                Attributes at1 = (Attributes) entries1.get(entryName);
                Attributes at2 = (Attributes) entries2.get(entryName);
                compareAttributes(diffUnit, entryName, at1, at2, processor);
            }
        } catch (IOException ioe) {
            log.error("Failure comparing manifest " + diffUnit.getScopedPath(), ioe);
        }
        return null; // No child scopes needed
    }

    /**
     * Performs a manifest comparison for a single manifest entry
     * 
     * @param diffUnit the unit to process
     * @param entryName the name of the manifest entry
     * @param at1 the attributes in the entry on the left; may be null
     * @param at2 the attributes in the entry on the right; may be null
     * @param processor the processor to pass differences to
     */
    private void compareAttributes(DiffUnit diffUnit, String entryName, Attributes at1,
            Attributes at2, DiffPointProcessor processor) {
        if (at1 == null) {
            String msg = "Manifest entry only in right: " + entryName;
            DiffPoint diffPoint = new DiffPoint(diffUnit, msg);
            processor.processDiffPoint(diffPoint);
        } else if (at2 == null) {
            String msg = "Manifest entry only in left: " + entryName;
            DiffPoint diffPoint = new DiffPoint(diffUnit, msg);
            processor.processDiffPoint(diffPoint);
        } else {
            SortedSet<Attributes.Name> attrNames = new TreeSet<Attributes.Name>(
                    new AttributesNameComparator());
            for (Object obj : at1.keySet()) {
                attrNames.add((Attributes.Name) obj);
            }
            for (Object obj : at2.keySet()) {
                attrNames.add((Attributes.Name) obj);
            }
            for (Iterator<Attributes.Name> it = attrNames.iterator(); it.hasNext();) {
                Attributes.Name attrName = it.next();
                String val1 = at1.getValue(attrName);
                String val2 = at2.getValue(attrName);
                String attrId = entryName + "." + attrName;
                if (val1 == null) {
                    String msg = "Manifest attribute only in right: " + attrId;
                    DiffPoint diffPoint = new DiffPoint(diffUnit, msg);
                    processor.processDiffPoint(diffPoint);
                } else if (val2 == null) {
                    String msg = "Manifest attribute only in left: " + attrId;
                    DiffPoint diffPoint = new DiffPoint(diffUnit, msg);
                    processor.processDiffPoint(diffPoint);
                } else if (!val1.equals(val2)) {
                    String msg = "Manifest attribute value mismatch for " + attrId + ": " + val1
                            + ", " + val2;
                    DiffPoint diffPoint = new DiffPoint(diffUnit, msg);
                    processor.processDiffPoint(diffPoint);
                }
            }
        }
    }

    /**
     * Comparator for {@link Attributes.Name}
     */
    private static final class AttributesNameComparator implements Comparator<Attributes.Name>,
            Serializable {
        private static final long serialVersionUID = 1L;

        /**
         * Compares two {@link Attributes.Name}s using their names (case insensitive)
         * 
         * @param n1 the first Name to compare
         * @param n2 the second Name to compare
         * 
         * @return the comparison result
         */
        public int compare(Attributes.Name n1, Attributes.Name n2) {
            String nStr1 = n1.toString();
            String nStr2 = n2.toString();
            return nStr1.compareToIgnoreCase(nStr2);
        }
    }
}
